#!/bin/bash

function splitBvh {
  echo "Splitting $1"

  # Find the MOTION section of the BVH file
  lineNumber=`sed -n '/^MOTION$/=' "$1"`
  hierarchyLastLineNumber=`expr $lineNumber - 1`

  lineNumber=`expr $lineNumber + 1`

  # Get the BVH file's frame count
  frameCount=`sed -n "$lineNumber "'s/^Frames: *\([0-9]*\)$/\1/p' "$1"`
  if [ -z "$frameCount" ]
  then
    echo "ERROR1: BVH file may be corrupted"
    return 1
  fi

  lineNumber=`expr $lineNumber + 1`

  # Get the BVH file's frame time
  frameTime=`sed -n "$lineNumber "'s/^Frame Time: *\([0-9.]*\)$/\1/p' "$1"`
  if [ -z "$frameTime" ]
  then
    echo "ERROR2: BVH file may be corrupted"
    return 2
  fi

  lineNumber=`expr $lineNumber + 1`

  # Get the first frame of the animation
  referenceFrame=`sed -n "${lineNumber}p" "$1"`

  lineNumber=`expr $lineNumber + 1`

  # Determine how many frames can be in each file to create 60-second animations
  framesPerFile=`echo 60 / $frameTime | bc`
  framesPerFile=`expr $framesPerFile - 1`

  if [ $framesPerFile -le 0 ]
  then
    echo "ERROR: BVH animation frame time is too large to be split"
    return 3
  fi

  # Initialize the counter for naming the split files
  newFrameCount=`expr $framesPerFile + 1`
  fc1=0
  fc2=0
  fc3=1
  fileCounter=$fc1$fc2$fc3
  remainingFramesCount=`expr $frameCount - 1`

  while [ $remainingFramesCount -gt 0 ]
  do
    newFile=`basename $1 .bvh`"${fileCounter}.bvh"
    if [ -w "$outputDir/$newFile" ]
    then
      rm -f "$outputDir/$newFile"
    fi

    # write beginning part of file
    sed -n "1,${hierarchyLastLineNumber}p" $1 >> $outputDir/$newFile
    echo "MOTION" >> $outputDir/$newFile

    # write frame metadata
    if [ $remainingFramesCount -lt $newFrameCount ]
    then
      newFrameCount=`expr $remainingFramesCount + 1`
    fi

    echo "Frames: $newFrameCount" >> $outputDir/$newFile
    echo "Frame Time: $frameTime" >> $outputDir/$newFile

    # write reference frame
    echo $referenceFrame >> $outputDir/$newFile

    # write rest of motion frames
    lastLine=`expr $lineNumber + $newFrameCount - 2`
    sed -n "${lineNumber},${lastLine}p" "$1" >> $outputDir/$newFile

    lineNumber=`expr $lastLine + 1`
    remainingFramesCount=`expr $remainingFramesCount - $newFrameCount + 1`

    # increment file counter
    if [ $fc3 -eq 9 ]
    then
      if [ $fc2 -eq 9 ]
      then
        fc1=`expr $fc1 + 1`
        fc2=0
      else
        fc2=`expr $fc2 + 1`
      fi
    else
      fc3=`expr $fc3 + 1`
    fi
    fileCounter=$fc1$fc2$fc3
  done
}


# Create the output directory
outputDir=splitter_output
if [ ! -e "$outputDir" ]
then
  mkdir "$outputDir"
fi

# Run this script on each BVH file in this directory
wereBvhFilesFound=
for file in *.bvh
do
  wereBvhFilesFound=true
  splitBvh "$file"
done

if [ -n "$wereBvhFilesFound" ]
then
  echo Done
  echo ""
  echo Split BVH files are in folder "$outputDir"
else
  echo No BVH files were found in the current folder
fi
echo ""
read -p "Press [Enter] to continue . . ."

